//
//  CakeMobileConversions.m
//  CakeSDKLessConversionCode
//
//  Created by Ryan Newsome on 11/29/12.
//  Copyright (c) 2014 Cake All rights reserved.
//

#import "CakeMobileConversions.h"
#import "Reachability.h"

#define SENT_EVENTS_FILE_NAME @"CakeSentEvents.plist"
#define CONVERSION_REQUEST_ID_FILE_NAME @"CakeRequestIDs.plist"

#define REQUEST_ID_KEY @"REQUEST_ID"
#define OFFER_ID_KEY @"OFFER_ID"

@interface CakeMobileConversions ()
{
    NSXMLParser *_conversionParser;
    NSXMLParser *_eventParser;
    NSMutableString *_code;
    NSMutableString *_message;
    NSMutableString *_requestSessionID;
    NSMutableString *_offerID;
    NSMutableString *_currentElement;
}

@property(nonatomic,assign) BOOL isDebug;
@property(nonatomic,strong) NSMutableData *responseData;
@property(nonatomic,strong) NSMutableArray *keyArray;
@property(nonatomic,strong) NSURLConnection *myConnection;
@property(nonatomic,strong) NSURLRequest *eventRequest;
@property(nonatomic,strong) NSURLRequest *conversionRequest;

@end

@implementation CakeMobileConversions

- (id)init
{
    self = [super init];
    if (self) {
        _isDebug = NO;
    }

    return self;
}

static CakeMobileConversions *_myInstance = nil;

+ (CakeMobileConversions*)sharedInstance
{
    if(_myInstance == nil)
    {
        _myInstance = [[CakeMobileConversions alloc]init];
    }

    return _myInstance;
}

#pragma mark public facing functions

+ (void)sendInstallConversionWithURLString:(NSString*)URLString
{
    Reachability *reachability = [Reachability reachabilityForInternetConnection];
    
    //Check if we have internet and then check if we haven't sent the install yet
    if([[CakeMobileConversions sharedInstance] isDebug] ||
       ([reachability currentReachabilityStatus] != NotReachable && ![CakeMobileConversions doesStringExistInSentURLs:URLString]))
    {
        [[CakeMobileConversions sharedInstance] sendURL:URLString isEvent:NO];
    }
}

+ (void)sendEventWithURLString:(NSString*)URLString doesAllowDuplicates:(BOOL)allowDuplicates
{
    Reachability *reachability = [Reachability reachabilityForInternetConnection];
    
    if([[CakeMobileConversions sharedInstance] isDebug] ||
       ([reachability currentReachabilityStatus] != NotReachable && ((!allowDuplicates && ![CakeMobileConversions doesStringExistInSentURLs:URLString]) || allowDuplicates)))
    {
        [[CakeMobileConversions sharedInstance] sendURL:URLString isEvent:YES];
    }
}

+ (void)setDebugMode:(BOOL)isDebug
{
    [CakeMobileConversions sharedInstance].isDebug = isDebug;
}

+ (void)addConversionKey:(NSString*)keyString
{
    [[CakeMobileConversions sharedInstance] addKeyString:keyString];
}

#pragma mark utility functions

- (void)addKeyString:(NSString*)key
{
    if(_keyArray == nil)
    {
        _keyArray = [[NSMutableArray alloc] init];
    }

    [_keyArray addObject:key];
}

+ (NSString*)buildURLFromBaseURLString:(NSString*)baseURL
{
    NSMutableString *finalUrl = [NSMutableString stringWithString:baseURL];

    if([[CakeMobileConversions sharedInstance] keyArray] != nil)
    {
        for(NSString *key in [[CakeMobileConversions sharedInstance] keyArray])
        {
            [finalUrl appendFormat:@"&k=%@", key];
        }
    }
    
    if([finalUrl rangeOfString:@"f="].location == NSNotFound)
    {
        [finalUrl appendFormat:@"&f=mpb"];
    }
    else
    {
        //if the string contains a postback format replace that with a mobile postback format
        if([finalUrl rangeOfString:@"f=pb"].location != NSNotFound)
        {
            finalUrl = [[NSMutableString alloc] initWithString:[finalUrl stringByReplacingOccurrencesOfString:@"f=pb" withString:@"f=mpb"]];
        }
    }

    //put in the version of the app
    NSString *version = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];

    //I don't think a valid app can be without this
    if(version!= nil)
    {
        [finalUrl appendFormat:@"&mv=%@",[version stringByReplacingOccurrencesOfString:@"." withString:@"_"]];
    }

    return finalUrl;
}

+ (NSArray*)getListOfSentURLs
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:SENT_EVENTS_FILE_NAME];

    if([[NSFileManager defaultManager] fileExistsAtPath:filePath])
    {
        return [NSArray arrayWithContentsOfFile:filePath];
    }
    else
    {
        return nil;
    }
}

+ (void)saveURLStringAsSent:(NSString*)urlString
{
    NSMutableArray *currentArrayURL = [[NSMutableArray alloc] initWithArray:[CakeMobileConversions getListOfSentURLs]];

    for(NSString *curURL in currentArrayURL)
    {
        if([urlString isEqualToString:curURL])
        {
            return;
        }
    }
    
    [currentArrayURL addObject:urlString];

    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:SENT_EVENTS_FILE_NAME];
    
    [currentArrayURL writeToFile:filePath atomically:YES];
}

+ (NSArray*)getStoredRequestIDs
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:CONVERSION_REQUEST_ID_FILE_NAME];
    
    if([[NSFileManager defaultManager] fileExistsAtPath:filePath])
    {
        return [NSArray arrayWithContentsOfFile:filePath];
    }
    else
    {
        return nil;
    }
}

+ (void)saveRequestIDString:(NSString*)requestID withOfferIDString:(NSString*)offerString
{
    NSMutableArray *currentArrayURL = [[NSMutableArray alloc] initWithArray:[CakeMobileConversions getStoredRequestIDs]];
 
    BOOL isFoundAlready = NO;
    
    for(NSDictionary *item in currentArrayURL)
    {
        if([[item objectForKey:REQUEST_ID_KEY] isEqualToString:requestID] && [[item objectForKey:OFFER_ID_KEY] isEqualToString:offerString])
        {
            isFoundAlready = YES;
            break;
        }
    }
    
    if(!isFoundAlready)
    {
        [currentArrayURL addObject:[NSDictionary dictionaryWithObjectsAndKeys:requestID, REQUEST_ID_KEY, offerString, OFFER_ID_KEY, nil]];

        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *documentsDirectory = [paths objectAtIndex:0];
        NSString *filePath = [documentsDirectory stringByAppendingPathComponent:CONVERSION_REQUEST_ID_FILE_NAME];
    
        [currentArrayURL writeToFile:filePath atomically:YES];
    }
}

+ (BOOL)doesStringExistInSentURLs:(NSString*)urlToCheck
{
    NSArray *currentURLS = [CakeMobileConversions getListOfSentURLs];
    BOOL foundURL = NO;

    if(currentURLS != nil)
    {
        for(NSString *url in currentURLS)
        {
            if([urlToCheck isEqualToString:url])
            {
                foundURL = YES;
            }
        }
    }

    if(foundURL)
    {
        return YES;
    }
    else
    {
        [CakeMobileConversions saveURLStringAsSent:urlToCheck];
        return NO;
    }
}

- (void)sendURL:(NSString*)urlString isEvent:(BOOL)isEvent
{
    NSURL *finalURL = [NSURL URLWithString:[CakeMobileConversions buildURLFromBaseURLString:[self getRequIDAddedURLWithBaseURLString:urlString]]];

    NSURLRequest *request = nil;
    
    if(isEvent)
    {
        _eventRequest = [NSURLRequest requestWithURL:finalURL];
        request = _eventRequest;
    }
    else
    {
        _conversionRequest = [NSURLRequest requestWithURL:finalURL];
        request = _conversionRequest;
    }
    
    if(_isDebug)
    {
        NSLog(@"Final URL to be sent: %@", finalURL);
    }
    
    _responseData = [[NSMutableData alloc] init];
    _myConnection = [[NSURLConnection alloc] initWithRequest:request delegate:self];
}

- (NSString*)getRequIDAddedURLWithBaseURLString:(NSString*)urlString
{
    NSString *existingReqID = nil;
    
    NSArray *comp1 = [urlString componentsSeparatedByString:@"?"];
    NSString *query = [comp1 lastObject];
    NSArray *queryElements = [query componentsSeparatedByString:@"&"];
    for (NSString *element in queryElements) {
        NSArray *keyVal = [element componentsSeparatedByString:@"="];
        if (keyVal.count > 0)
        {
            if([[keyVal objectAtIndex:0] isEqualToString:@"o"])
            {
                _offerID = (keyVal.count == 2) ? [keyVal lastObject] : nil;
            }
            else if([[keyVal objectAtIndex:0] isEqualToString:@"r"])
            {
                existingReqID = (keyVal.count == 2) ? [keyVal lastObject] : nil;
            }
        }
    }
    
    if(_offerID != nil)
    {
        NSString *reqID = [self getReqIDAddedURLWithOfferID:_offerID];
        
        if(reqID != nil)
        {
            if(existingReqID == nil)
            {
                urlString = [NSString stringWithFormat:@"%@&r=%@", urlString, reqID];
            }
            else
            {
                urlString = [urlString stringByReplacingOccurrencesOfString:existingReqID withString:reqID];
            }
        }
    }
    
    return urlString;
}

- (NSString*)getReqIDAddedURLWithOfferID:(NSString*)offerIDString
{
    for(NSDictionary *reqIDDict in [CakeMobileConversions getStoredRequestIDs])
    {
        if([[reqIDDict objectForKey:OFFER_ID_KEY] isEqualToString:offerIDString])
        {
            return [reqIDDict objectForKey:REQUEST_ID_KEY];
        }
    }
    
    return nil;
}

#pragma mark connection delegate functions

- (void)connection:(NSURLConnection *) connection didReceiveData:(NSData *)data
{
    [_responseData appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    NSString *response = [[NSString alloc] initWithData:_responseData encoding:NSASCIIStringEncoding];
    
    if(_isDebug && response != nil)
    {
        NSLog(@"Server response: %@", response);
    }

    if(response != nil && [connection originalRequest] == _eventRequest)
    {
        _eventParser = [[NSXMLParser alloc] initWithData:[response dataUsingEncoding:NSUTF8StringEncoding]];
        [_eventParser setDelegate:self];
        [_eventParser setShouldProcessNamespaces:NO];
        [_eventParser setShouldReportNamespacePrefixes:NO];
        [_eventParser setShouldResolveExternalEntities:NO];
        [_eventParser parse];
    }
    else
    {
        _conversionParser = [[NSXMLParser alloc] initWithData:[response dataUsingEncoding:NSUTF8StringEncoding]];
        [_conversionParser setDelegate:self];
        [_conversionParser setShouldProcessNamespaces:NO];
        [_conversionParser setShouldReportNamespacePrefixes:NO];
        [_conversionParser setShouldResolveExternalEntities:NO];
        [_conversionParser parse];
    }

    _myConnection = nil;
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    _myConnection = nil;
}

#pragma mark xml parsing

- (void)parserDidStartDocument:(NSXMLParser *)parser
{
}

- (void)parserDidEndDocument:(NSXMLParser *)parser
{
    if(parser == _eventParser)
    {
        if([_code isEqualToString:@"0"] && _requestSessionID != nil)
        {
            [CakeMobileConversions saveRequestIDString:_requestSessionID withOfferIDString:_offerID];
        }
    }
    else if(parser == _conversionParser)
    {
        if([_code isEqualToString:@"0"] && _requestSessionID != nil)
        {
            [CakeMobileConversions saveRequestIDString:_requestSessionID withOfferIDString:_offerID];
        }
    }
}

- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName
  namespaceURI:(NSString *)namespaceURI
 qualifiedName:(NSString *)qName
    attributes:(NSDictionary *)attributeDict
{
    if([elementName isEqualToString:@"code"])
    {
        _code = [[NSMutableString alloc] init];
        _currentElement = _code;
    }
    else if([elementName isEqualToString:@"msg"])
    {
        _message = [[NSMutableString alloc] init];
        _currentElement = _message;
    }
    else if([elementName isEqualToString:@"request_session_id"])
    {
        _requestSessionID = [[NSMutableString alloc] init];
        _currentElement = _requestSessionID;
    }
    else if([elementName isEqualToString:@"offer_id"])
    {
        _offerID = [[NSMutableString alloc] init];
        _currentElement = _offerID;
    }
}

- (void)parser:(NSXMLParser *)parser
 didEndElement:(NSString *)elementName
  namespaceURI:(NSString *)namespaceURI
 qualifiedName:(NSString *)qName;
{
    _currentElement = nil;
}

- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    [_currentElement appendString:[string stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]]];
}

- (void)parser:(NSXMLParser *)parser parseErrorOccurred:(NSError *)parseError
{
    _code = _message = _requestSessionID = _offerID = nil;
}

@end
